#
# Copyright (C) 2002 Jp Calderone.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of version 2.1 of the GNU Lesser General Public
# License as published by the Free Software Foundation.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""
This module provides two filters and three commands.

The first filter simply ignores users it is directed to do so.  The second
tries to notice when two bridge bots have gone into a loop with each other,
and add that bot to the ignore list.

The three commands deal with managing ignore lists manually, both for users
on the local network and for users being bridged from remote ones.
"""

# System imports
import re

# Pynfo imports
from Pynfo import interfaces, error

# Twisted imports
from twisted.python import components

class IDiscriminating(components.Interface):
    def disregard(self, network, name):
        """
        Add the given name on the given network to the list of users
        to be ignored.
        """
    def regard(self, network, name):
        """
        Remove the given name on the given network from the list of users
        to be ignored.
        """
    def isDisregarded(self, network, name):
        """
        Check if the given name on the given network is being disregarded.
        """

class DiscriminatingAdapter(components.Adapter):
    __implements__ = (IDiscriminating,)

    ignoreTable = None
    def __init__(self, original):
        components.Adapter.__init__(self, original)
        self.ignoreTable = {}

    def disregard(self, network, name):
        self.ignoreTable.setdefault(network, {})[name] = True

    def regard(self, network, name):
        del self.ignoreTable[network][name]

    def isDisregarded(self, network, name):
        return self.ignoreTable.setdefault(network, {}).has_key(name)

def init(factory):
    # Register the adapter if necessary
    if factory.getComponent(DiscriminatingAdapter) is None:
        factory.setAdapter(IDiscriminating, DiscriminatingAdapter)


def filter_IGNORE(bot, user, channel, line):
    ignore = bot.factory.getComponent(IDiscriminating)
    if ignore.isDisregarded(bot.network.alias, user):
        return 'swallow'


# Prevent infinite bot recursions
bridge = re.compile('<\w+@\w+>')
def filter_AUTOIGNORE(bot, user, channel, line):
    ignore = bot.factory.getComponent(IDiscriminating)
    if len(bridge.findall(line)) >= 3:
        bot.reply(channel, "Auto-ignoring " + user)
        ignore.disregard(bot.network.alias, user)
        return 'swallow'


def privinfo_IGNORE(bot, user, channel, args):
    """Usage: ignore <user>[@<network>] - Ignore users matching the given pattern.  If <network> is given, the ignore rule is only applied to users on that network."""
    ignore = bot.factory.getComponent(IDiscriminating)
    info = args.split('@', 1)
    if len(info) == 1:
        if args in bot.network.owners:
            bot.reply(channel, "You cannot ignore an owner.")
        elif ignore.isDisregarded(bot.network.alias, args):
            bot.reply(channel, "He is already ignored.")
        else:
            ignore.disregard(bot.network.alias, info[0])
            bot.reply(channel, "Ignored %r." % args)
    else:
        target, network = info
        try:
            bot = bot.factory.allBots[network]
        except KeyError:
            bot.reply(channel, "I don't know what network that is.")
        else:
            privinfo_IGNORE(bot, user, channel, target)


def privinfo_UNIGNORE(bot, user, channel, args):
    """Usage: unignore <user>[@<network>] - Remove the given pattern from the ignore list"""
    ignore = bot.factory.getComponent(IDiscriminating)
    info = args.split('@', 1)
    if len(info) == 1:
        try:
            ignore.regard(bot.network.alias, info[0])
        except KeyError:
            bot.reply(channel, "That person is not ignored.")
        else:
            bot.reply(channel, "Unignored %r." % args)
    else:
        target, network = info
        try:
            bot = bot.factory.allBots[network]
        except KeyError:
            bot.reply(channel, "I don't know what network that is.")
        else:
            privinfo_UNIGNORE(bot, user, channel, target)


def info_IGNORES(bot, user, channel, args):
    """Usage: ignores - List all ignored patterns"""
    table = bot.factory.getComponent(IDiscriminating).ignoreTable
    ignores = ['@'.join((u, n)) for n in table for u in table[n]]
    if ignores:
        bot.reply(channel, "Ignoring: %s" % ' '.join(ignores))
    else:
        bot.reply(channel, "No one is being ignored just now.")
