#
#   Twisted Pynfo - A relay, info, and search bot for IRC.
#   Copyright (C) 2002 Jp Calderone (exarkun@twistedmatrix.com)
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
#

import traceback, os, socket

from twisted.spread import pb
from twisted.python import usage
from twisted.internet import reactor
from twisted.web import vhost, server, widgets, distrib
import twisted.python.util

import util, info, shorter, logview, adapters

class Options(usage.Options):
    synopsis = "Usage: mktap infobot [options]"
    
    optParameters = [
        ["nick", "n", "pynfo", "Bot nickname"],
        ["server", "s", "irc.freenode.net", "IRC server hostname"],
        ["port", "p", "6667", "IRC server port"],
        ["alias", "a", "fn", "IRC network alias"],
        ["datadir", None, "info.db", "Path to place database files"],
        ["bits", "b", 128, "Bits in secure command RSA key"],

        ["webhost", "h", socket.getfqdn(), "Hostname on which to listen for http queries"],
        ["webport", "w", None, "Port to listen on for Shorter Link connections"],
        ["distport", "d", None, "Port to on which to publish distributed resources"],
    ]

    optFlags = [
        ["pwdauth", None, "Use password authentication"],
    ]

    def __init__(self):
        usage.Options.__init__(self)

        self.channels = util.CaseList()
        self.owners = util.CaseList()
        self.plugins = []


    def opt_channel(self, channel):
        """The name of a channel to join upon connecting to the server"""
        if not channel.startswith('#'):
            channel = '#' + channel
        self.channels.append(channel.lower())
    
    
    def opt_owner(self, owner):
        """The IRC nickname of a user to grant special access"""
        self.owners.append(owner.lower())


    def verifyPort(self, key):
        try:
            self[key] = int(self[key])
        except ValueError:
            raise usage.UsageError("Invalid %s number: %s" % (key, self[key]))
        else:
            if self[key] < 0 or self[key] >= 2 ** 16:
                raise usage.UsageError("Invalid %s number: %d" % (key, self[key]))


    def postOptions(self):
        for k in 'port', 'webport', 'distport':
            if self[k]:
                self.verifyPort(k)

        try:
            self['bits'] = int(self['bits'])
        except ValueError:
            raise usage.UsageError("Invalid value for bits: " + self['bits'])
        else:
            if self['bits'] < 128:
                raise usage.UsageError("Value for bits must be >= 128")


def updateApplication(app, config):
    host = config['server']
    port = config['port']
    nick = config['nick']
    alias = config['alias']
    dbpath = config['datadir']
    pwdauth = config['pwdauth']

    if not os.path.isdir(dbpath):
        os.mkdir(dbpath)
    
    factory = info.InfoBotFactory(dbpath, bits=config['bits'])
    factory.createNetwork((host, port), alias, nick, config.owners, config.channels)

    if pwdauth:
        n = factory.networkByAlias(alias)
        factory.setAuthenticator(adapters.PasswordAuthenticator)
        for o in config.owners:
            p = twisted.python.util.getPassword('Password for ' + o + ': ')
            n.passwords[o] = p
        factory.setNetwork(n)
    else:
        factory.setAuthenticator(adapters.ServicesAuthenticator)

    app.connectTCP(host, port, factory)

    if config['distport'] or config['webport']:
        convo = logview.ChannelViewer(factory)
        short = shorter.Shortener(factory)
        vh = vhost.NameVirtualHost()
        vh.addHost('l.' + config['webhost'], short)
        vh.addHost('c.' + config['webhost'], convo)
        site = server.Site(vh)
        if config['distport']:
            app.listenTCP(int(config['distport']), pb.BrokerFactory(distrib.ResourcePublisher(site)))
        if config['webport']:
            app.listenTCP(int(config['webport']), site)
            factory.webhost = config['webhost']
            factory.webport = config['webport']
